unit GroupCheckBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;
type

  TCustomCheckGroup = class(TCustomGroupBox)
  private
	 FButtons: TList;
	 FItems: TStrings;
	 FItemIndex: Integer;
	 FColumns: Integer;
	 FReading: Boolean;
	 FUpdating: Boolean;
	 procedure ArrangeButtons;
	 procedure ButtonClick(Sender: TObject);
	 procedure ItemsChange(Sender: TObject);
	 procedure SetButtonCount(Value: Integer);
	 procedure SetColumns(Value: Integer);
	 procedure SetItemIndex(Value: Integer);
	 procedure SetItems(Value: TStrings);
	 procedure UpdateButtons;
	 procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
	 procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
	 procedure WMSize(var Message: TWMSize); message WM_SIZE;
 	  Function GetCheckedStatus(Index:Integer):Boolean;
	  Procedure SetCheckedStatus(Index:Integer;Value:Boolean);
	  Function GetSelCount:integer;

  protected
	 procedure ReadState(Reader: TReader); override;
	 function CanModify: Boolean; virtual;
	 procedure GetChildren(Proc: TGetChildProc); override;
	 property Columns: Integer read FColumns write SetColumns default 1;
	 property ItemIndex: Integer read FItemIndex write SetItemIndex default -1;
	 property Items: TStrings read FItems write SetItems;
  public
	 property Checked[Index: Integer]: Boolean read GetCheckedStatus write SetCheckedStatus;
	 property SelCount:Integer read  GetSelCount;
	 constructor Create(AOwner: TComponent); override;
	 destructor Destroy; override;
  end;

  TCheckGroup = class(TCustomCheckGroup)
  published
	 property Align;
	 property Caption;
	 property Color;
	 property Columns;
	 property Ctl3D;
	 property DragCursor;
	 property DragMode;
	 property Enabled;
	 property Font;
	 property ItemIndex;
	 property Items;
	 property ParentColor;
	 property ParentCtl3D;
	 property ParentFont;
	 property ParentShowHint;
	 property PopupMenu;
	 property ShowHint;
	 property TabOrder;
	 property TabStop;
	 property Visible;
	 property OnClick;
	 property OnDragDrop;
	 property OnDragOver;
	 property OnEndDrag;
	 property OnEnter;
	 property OnExit;
	 property OnStartDrag;
  end;

procedure Register;


implementation

uses Consts;

{ TGroupButton }

type
  TGroupButton = class(TCheckBox)
  private
	 FInClick: Boolean;
	 procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
  protected
	 procedure ChangeScale(M, D: Integer); override;
	 procedure KeyDown(var Key: Word; Shift: TShiftState); override;
	 procedure KeyPress(var Key: Char); override;
  public
	 constructor Create(CheckGroup: TCustomCheckGroup);
	 destructor Destroy; override;
  end;

constructor TGroupButton.Create(CheckGroup: TCustomCheckGroup);
begin
  inherited Create(CheckGroup);
  CheckGroup.FButtons.Add(Self);
  Visible := False;
  Enabled := CheckGroup.Enabled;
  ParentShowHint := False;
  OnClick := CheckGroup.ButtonClick;
  Parent := CheckGroup;
end;

destructor TGroupButton.Destroy;
begin
  TCustomCheckGroup(Owner).FButtons.Remove(Self);
  inherited Destroy;
end;

procedure TGroupButton.CNCommand(var Message: TWMCommand);
begin
  if not FInClick then
  begin
    FInClick := True;
    try
      if ((Message.NotifyCode = BN_CLICKED) or
        (Message.NotifyCode = BN_DOUBLECLICKED)) and
		 TCustomCheckGroup(Parent).CanModify then
        inherited;
    except
      Application.HandleException(Self);
    end;
    FInClick := False;
  end;
end;

procedure TGroupButton.ChangeScale(M, D: Integer);
begin
end;

procedure TGroupButton.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  TCustomCheckGroup(Parent).KeyPress(Key);
  if (Key = #8) or (Key = ' ') then
  begin
    if not TCustomCheckGroup(Parent).CanModify then Key := #0;
  end;
end;

procedure TGroupButton.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  TCustomCheckGroup(Parent).KeyDown(Key, Shift);
end;

{ TCustomCheckGroup }

constructor TCustomCheckGroup.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csSetCaption, csDoubleClicks];
  FButtons := TList.Create;
  FItems := TStringList.Create;
  TStringList(FItems).OnChange := ItemsChange;
  FItemIndex := -1;
  FColumns := 1;
end;

destructor TCustomCheckGroup.Destroy;
begin
  SetButtonCount(0);
  TStringList(FItems).OnChange := nil;
  FItems.Free;
  FButtons.Free;
  inherited Destroy;
end;

procedure TCustomCheckGroup.ArrangeButtons;
var
  ButtonsPerCol, ButtonWidth, ButtonHeight, TopMargin, I: Integer;
  DC: HDC;
  SaveFont: HFont;
  Metrics: TTextMetric;
begin
  if (FButtons.Count <> 0) and not FReading then
  begin
    DC := GetDC(0);
    SaveFont := SelectObject(DC, Font.Handle);
    GetTextMetrics(DC, Metrics);
	 SelectObject(DC, SaveFont);
    ReleaseDC(0, DC);
    ButtonsPerCol := (FButtons.Count + FColumns - 1) div FColumns;
    ButtonWidth := (Width - 10) div FColumns;
    I := Height - Metrics.tmHeight - 5;
    ButtonHeight := I div ButtonsPerCol;
    TopMargin := Metrics.tmHeight + 1 + (I mod ButtonsPerCol) div 2;
	 for I := 0 to FButtons.Count - 1 do
      with TGroupButton(FButtons[I]) do
	   begin
        SetBounds((I div ButtonsPerCol) * ButtonWidth + 8,
          (I mod ButtonsPerCol) * ButtonHeight + TopMargin,
		   ButtonWidth, ButtonHeight);
        Visible := True;
	   end;
  end;
end;

procedure TCustomCheckGroup.ButtonClick(Sender: TObject);
begin
  if not FUpdating then
  begin
	 FItemIndex := FButtons.IndexOf(Sender);
    Click;
  end;
end;

procedure TCustomCheckGroup.ItemsChange(Sender: TObject);
begin
  if not FReading then
  begin
    if FItemIndex >= FItems.Count then FItemIndex := FItems.Count - 1;
    UpdateButtons;
  end;
end;

procedure TCustomCheckGroup.ReadState(Reader: TReader);
begin
  FReading := True;
  inherited ReadState(Reader);
  FReading := False;
  UpdateButtons;
end;

procedure TCustomCheckGroup.SetButtonCount(Value: Integer);
begin
  while FButtons.Count < Value do TGroupButton.Create(Self);
  while FButtons.Count > Value do TGroupButton(FButtons.Last).Free;
end;

procedure TCustomCheckGroup.SetColumns(Value: Integer);
begin
  if Value < 1 then Value := 1;
  if Value > 16 then Value := 16;
  if FColumns <> Value then
  begin
	 FColumns := Value;
    ArrangeButtons;
  end;
end;

procedure TCustomCheckGroup.SetItemIndex(Value: Integer);
begin
  if FReading then FItemIndex := Value
  else
  begin
	 if Value < -1 then Value := -1;
	 if Value >= FButtons.Count then Value := FButtons.Count - 1;
	 if Value >= 0 then TGroupButton(FButtons[Value]).Checked := True;
	 FItemIndex:=Value;
  end;
end;


Function TCustomCheckGroup.GetCheckedStatus(Index:Integer):Boolean;
begin
	   GetCheckedStatus:=TGroupButton(FButtons[Index]).Checked;
end;

Procedure TCustomCheckGroup.SetCheckedStatus(Index:Integer;Value:Boolean);
begin
	TGroupButton(FButtons[Index]).Checked:=Value;
end;

 Function TCustomCheckGroup.GetSelCount:integer;
var
	ix:integer;
	sum:integer;
begin
	sum:=0;
	for ix:= 0 to FButtons.Count-1 do
		if TGroupButton(FButtons[ix]).Checked then sum:=sum + 1;
	GetSelCount:=sum;
end;


procedure TCustomCheckGroup.SetItems(Value: TStrings);
begin
  FItems.Assign(Value);
end;

procedure TCustomCheckGroup.UpdateButtons;
var
  I: Integer;
begin
  SetButtonCount(FItems.Count);
  for I := 0 to FButtons.Count - 1 do
	 TGroupButton(FButtons[I]).Caption := FItems[I];
  if FItemIndex >= 0 then
  begin
	 FUpdating := True;
	 TGroupButton(FButtons[FItemIndex]).Checked := True;
    FUpdating := False;
  end;
  ArrangeButtons;
end;

procedure TCustomCheckGroup.CMEnabledChanged(var Message: TMessage);
var
  I: Integer;
begin
  inherited;
  for I := 0 to FButtons.Count - 1 do
	 TGroupButton(FButtons[I]).Enabled := Enabled;
end;

procedure TCustomCheckGroup.CMFontChanged(var Message: TMessage);
begin
  inherited;
  ArrangeButtons;
end;

procedure TCustomCheckGroup.WMSize(var Message: TWMSize);
begin
  inherited;
  ArrangeButtons;
end;

function TCustomCheckGroup.CanModify: Boolean;
begin
  Result := True;
end;

procedure TCustomCheckGroup.GetChildren(Proc: TGetChildProc);
begin
end;



procedure Register;
begin
  RegisterComponents('Samples', [TCheckGroup]);
end;

end.
